#ifndef ADC_HEADER
#define ADC_HEADER

#include <avr/io.h>

// define stuff (buggy avr-libc headers)
#ifndef ACMPM
	#define ACMPM ADC3D
#endif
#ifndef ACMP2D
	#define ACMP2D ADC2D
#endif
#ifndef ACMP1D
	#define ACMP1D ADC10D
#endif

/*
	Enable + division factor 8
	-> 2MHz clock
	This way it takes a little under 10us to make a conversion
*/
//#define ADCSRA_SETTINGS ((1<<ADEN)|(1<<ADPS1)|(1<<ADPS0))
// clk/64
#define ADCSRA_SETTINGS ((1<<ADEN)|(1<<ADPS2)|(1<<ADPS1))

// 5V, left-adjust
#define ADMUX_SETTINGS ((1<<REFS0) | (1<<ADLAR))

#define CH_VOLTAGE 8
#define CH_TEMPERATURE 9
#define CH_CURRENT 11 //AMP0
#define CH_SERVO1 6
#define CH_ST 1 // SENSOR:ST, ISP:CLK
#define CH_S2 0 // SENSOR:S2

inline void init_adc(void)
{
	/*
		ADC0: CURRENT
		ADC8: VDD_MOTOR
	*/
	{
		// ADCSRA - ADC Control and Status Register A
		ADCSRA = ADCSRA_SETTINGS | (1<<ADIF);
	}
	{
		// ADCSRB - ADC Control and Status Register B
		uint8_t b = 0;
		// ADC High Speed Mode:
		// Enable
		b |= (1<<ADHSM);
		// ADC Auto Trigger Source Selection Bits:
		// ADTS3:0: See datasheet.
		ADCSRB = b;
	}
	
	// Start a voltage conversion
	ADMUX = ADMUX_SETTINGS | CH_VOLTAGE;
	ADCSRA = ADCSRA_SETTINGS | (1<<ADIF) | (1<<ADSC);
}

inline void init_analog(void)
{
	{
		// DIDR0 - Digital Input Disable Register 0
		uint8_t b = 0;
		b |= (1<<ADC0D); // SENSOR:S2
		//b |= (1<<ADC1D); // SENSOR:ST, ISP:CLK
		b |= (1<<ACMPM); // VSC_3 (GND)
		b |= (1<<ACMP2D); // VS3
		b |= (1<<AMP0PD); // I_SENSE
		b |= (1<<AMP0ND); // I_SENSE_GND
		DIDR0 = b;
	}
	{
		// DIDR1 - Digital Input Disable Register 1
		uint8_t b = 0;
		b |= (1<<ACMP0D); // VS1
		b |= (1<<ACMP1D); // VS2
		b |= (1<<ADC9D); // TEMPERATURE
		b |= (1<<ADC8D); // VOLTAGE
		DIDR1 = b;
	}
	{
		// AMP0CSR - Amplifier 0 Control Register
		uint8_t b = 0;
		// Enable
		b |= (1<<AMP0EN);
		// Gain 40
		b |= (1<<AMP0G1);
		b |= (1<<AMP0G0);
		// Trig on PSC0_ASY
		b &=~(1<<AMP0TS1);
		b |= (1<<AMP0TS0);
		AMP0CSR = b;
	}
}

inline void init_comparator(void)
{
	/*
		We're using all three of the comparators and comparing each one to ACMPM
	*/
	{
		// AC0CON
		uint8_t b = 0;
		// Analog Comparator 0 Enable:
		// Do it
		b |= (1<<AC0EN);
		// Analog Comparator 0 Interrupt Enable:
		// Don't do it
		b &= ~(1<<AC0IE);
		// Analog Comparator 0 Interrupt Select:
		// AC0IS1 AC0IS0 Description
		// 0      0      Comparator Interrupt on output toggle
		// 0      1      Reserved
		// 1      0      Comparator Interrupt on output falling edge
		// 1      1      Comparator Interrupt on output rising edge
		b &= ~(1<<AC0IS1) & ~(1<<AC0IS0);
		// Analog Comparator 0 Multiplexer register:
		// AC0M2:0: See datasheet.
		// Analog Comparator Negative Input (ACMPM Pin)
		b |= (1<<AC0M2);
		b &= ~(1<<AC0M1) & ~(1<<AC0M0);
		AC0CON = b;
		AC1CON = b;
		AC2CON = b;
	}
	{
		// ACRS - Analog Comparator Status Register
		// The ACSR register has a clk/2 option for 3V operation (ACCKDIV),
		// but we are using 5V so there's no need to touch it.
	}
}

#endif

