/*
	This file is a part of the dcdrive2 firmware.
	(c) 2010 Perttu Ahola

	Modified version for the Tuomipäivä 666 robot.
*/

/*
    MCU: AT90PWM3B

    FUSE SETTINGS:

    used:
        efuse=0xf9 //avrdude seems to not be able to change this
        hfuse=0xce //WDT always on, 4.5V brown-out
        lfuse=0xd3 //disable ckdiv8, 4ms start-up delay, (RC->PLL)/4=16MHz
		
        //hfuse=0xcc //WDT always on, 4.3V brown-out
        //efuse=0x19 //PSC2,1,0 forced at high level after reset
        //hfuse=0xc9 //WDT always on, 2.8V brown-out?

    chip defaults:
        efuse=0xf9
        hfuse=0xdf
        lfuse=0x62

    random:
    TEMP = .448V in normal temp.


	|    0     |    1     |    2     |    3     |    4     |    5     |
	|          |          |          |          |          |          |
	|----------|xxxxxxxxxx|xxxxxxxxxx|----------|----------|----------|
	|       xxx|          |          |xxx       |          |          |
0:	|---xxxx---|----------|----------|---xxxx---|----------|----------|
	|xxx       |          |          |       xxx|          |          |
	|----------|----------|----------|----------|xxxxxxxxxx|xxxxxxxxxx|
	|          |          |          |          |          |          |
	|          |          |          |          |          |          |
	|----------|----------|----------|xxxxxxxxxx|xxxxxxxxxx|----------|
	|          |          |       xxx|          |          |xxx       |
1:	|----------|----------|---xxxx---|----------|----------|---xxxx---|
	|          |          |xxx       |          |          |       xxx|
	|xxxxxxxxxx|xxxxxxxxxx|----------|----------|----------|----------|
	|          |          |          |          |          |          |
	|          |          |          |          |          |          |
	|xxxxxxxxxx|----------|----------|----------|----------|xxxxxxxxxx|
	|          |xxx       |          |          |       xxx|          |
2:	|----------|---xxxx---|----------|----------|---xxxx---|----------|
	|          |       xxx|          |          |xxx       |          |
	|----------|----------|xxxxxxxxxx|xxxxxxxxxx|----------|----------|
	|          |          |          |          |          |          |


Operation:
-----------------

The PSCs are controlled so that two outputs are low and the third one is
floating for a specified period of time every PWM cycle.

The number of the floating PSC is stored in g_floating_psc.

Driver connections:
-----------------------

PSCOUTn1: Low Side
PSCOUTn0: High Side

How to use the Center Aligned Mode:

OCRnRB:
	The maximum value of the count. The cycle time is 2*this.
OCRnSB:
	When counting down, at this value, PSCOUTn1 is reset.
	When counting up, at this value, PSCOUTn1 is set.
	On AT90PWM2/3 the operation is inverted.
OCRnSA:
	When counting down, at this value, PSCOUTn0 is set.
	When counting up, at this value, PSCOUTn0 is reset.

PSCOUTn1 On-time = 1/clk * 2 * (OCRnRB - OCRnSB + 1)
PSCOUTn0 On-time = 1/clk * 2 * OCRnSA
Dead-time        = 1/clk * (OCRnSB - OCRnSA)
Cycle            = 1/clk * 2 * (OCRnRBH + 1)

cycletime = (rb + 1) * 2
lowtime = 2 * (rb - sb + 1)
hightime = 2 * sa
deadtime = sb - sa

=>

cycletime = 2 * rb + 2
<=> rb = cycletime / 2 - 1

lowtime / 2 = rb - sb + 1
<=> sb = cycletime / 2 - 1 - lowtime / 2 + 1
<=> sb = cycletime / 2 - lowtime / 2

lowtime / 2 = rb - sb + 1
<=> rb = lowtime / 2 + sb - 1

sa = hightime / 2

deadtime = sb - sa
<=> sa = sb - deadtime
	   = cycletime / 2 - lowtime / 2 - deadtime

deadtime = sb - sa
<=> sb = deadtime + sa
	   = deadtime + hightime / 2

=> rb = lowtime / 2 + deadtime + hightime / 2 - 1
	  = lowtime / 2 + hightime / 2 + deadtime - 1


Sensor configuration:

Stick forward = !g_reverse = clockwise (looked towards the motor)

High states when spinning forward:
Sensor 1 = PSC A
Sensor 2 = PSC C
Sensor 3 = PSC B
PSC high states when spinning forward:
A->C->B

*/

#include <avr/io.h>
// There's a bug in avr/io90pwm3b.h which defines WDE3 instead of WDE
#ifndef WDE
	#define WDE WDE3
#endif

#define F_CPU 16000000L
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <util/atomic.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "io.h"
#include "analog.h"

// =========================================================================
//                                                                     TYPES
// =========================================================================

enum ControlMode
{
	CONTROL_FIXED,
	CONTROL_SERVO2_RC,
	CONTROL_SERVO1_ANALOG,
	CONTROL_SERVO_MELTY,
	CONTROL_SERVO_MELTY_ACCEL, // Use acceleration sensor
};

enum ThrottleMode
{
	THROTTLE_ORIGINAL,
	THROTTLE_NEW
};

enum CommMode
{
	COMM_MAINLOOP,
	COMM_INTERRUPT,
	COMM_SENSOR,
	COMM_NONE,
	COMM_FET_TEST
};

enum BrakeMode
{
	BRAKE_ROBOT,
	BRAKE_CAR
};

enum LedMode
{
	LEDMODE_COMM0,
	LEDMODE_RX
};

/*
	Defines the parameters of a PSC channel
	
	SA controls the end of PSCOUTn0 (High Side) ON-time
	SB controls the start of PSCOUTn1 (Low Side) ON-time
*/
struct PscControl
{
	uint8_t sa;
	uint8_t sb;
};

// State of commutation at a point of time
struct comm_state
{
	uint8_t high;     // high PSC channel
	uint8_t low;      // low PSC channel
	uint8_t floating; // floating PSC channel
	uint8_t rising;   // edge direction of floating channel
};

// =========================================================================
//                                                                 CONSTANTS
// =========================================================================

#define TRUE 1
#define FALSE 0

const enum CommMode     c_comm_mode     = COMM_MAINLOOP;
//const enum CommMode     c_comm_mode     = COMM_INTERRUPT;
//const enum CommMode     c_comm_mode     = COMM_SENSOR;
//const enum CommMode     c_comm_mode     = COMM_NONE;
//const enum CommMode     c_comm_mode     = COMM_FET_TEST;

//#define CONTROL_MODE CONTROL_FIXED
#define CONTROL_MODE CONTROL_SERVO2_RC
//#define CONTROL_MODE CONTROL_SERVO_MELTY
//#define CONTROL_MODE CONTROL_SERVO_MELTY_ACCEL

const enum ThrottleMode c_throttle_mode = THROTTLE_NEW;

//const enum BrakeMode    c_brake_mode    = BRAKE_ROBOT;
const enum BrakeMode    c_brake_mode    = BRAKE_CAR;

const enum LedMode      c_ledmode       = LEDMODE_RX;

#define ENABLE_SERVO2_INPUT (\
	CONTROL_MODE == CONTROL_SERVO_MELTY ||\
	CONTROL_MODE == CONTROL_SERVO_MELTY_ACCEL ||\
	CONTROL_MODE == CONTROL_SERVO2_RC\
)

#define ENABLE_SERVO1_INPUT (\
	CONTROL_MODE == CONTROL_SERVO_MELTY ||\
	CONTROL_MODE == CONTROL_SERVO_MELTY_ACCEL\
)

#define ENABLE_MELTY (\
	CONTROL_MODE == CONTROL_SERVO_MELTY ||\
	CONTROL_MODE == CONTROL_SERVO_MELTY_ACCEL\
)

// If 1, some panics are enabled that otherwise wouldn't
#define DEBUG_MODE 0

// Current limit
//#define MAXIMUM_CURRENT 5
//#define MAXIMUM_CURRENT 10
//#define MAXIMUM_CURRENT 15 // assembly 2012 viimeinen kadotus
//#define MAXIMUM_CURRENT 20
//#define MAXIMUM_CURRENT 30
#define MAXIMUM_CURRENT 35
//#define MAXIMUM_CURRENT 45

//#define MAX_MOTOR_RPM 3500
//#define MAX_MOTOR_RPM 15000 // Used in "Viimeinen kadotus"
//#define MAX_MOTOR_RPM 17000
//#define MAX_MOTOR_RPM 20000
#define MAX_MOTOR_RPM 40000

// 1. rengas (pienempi hiirimatto, väri sisäpuolella)
//#define WHEEL_RPM_TO_ROBOT_RPM (25.0/47.0/6.0)
// 2. rengas (hitusen suurempi hiirimatto, väri ulkopuolella)
//#define WHEEL_RPM_TO_ROBOT_RPM (26.5/47.0/6.0)
// 2. rengas, eri osa-asettelu
#define WHEEL_RPM_TO_ROBOT_RPM (26.5/46.5/6.0)

// USART baud rate
//#define RS232_BAUD 19200
//#define RS232_BAUD 9600
#define RS232_BAUD 38400

// If fixed throttle is used, this is the throttle
#define FIXED_THROTTLE (PSC_RB/10)

// Size of servo pulse deadzone
#define RC_DEADZONE 10
//#define RC_DEADZONE 4

// Whether to use exponential throttle
#define EXP_THROTTLE 1

// If car mode is used, fast reverse does nothing
#define DISABLE_FAST_REVERSE (c_brake_mode == BRAKE_CAR)

// PSC channels
#define PSC_A 2 // T1
#define PSC_B 1 // T2
#define PSC_C 0 // T3
#define PSC_NONE 255

// Use PLL clock for PSCs
#define PSC_SELECTED_PCLKSEL (1<<PCLKSEL0)

// PSC 0 Center Aligned Mode + clock select (+output active low)
#define PCNF_SETTING ((1<<PMODE01)|(1<<PMODE00)|PSC_SELECTED_PCLKSEL)

// So we have this clock for the PCSs
#define PSC_CLOCK (32000000/4) // prescaler=4

// PSC deadtime
//#define PWM_DEADTIME ((uint16_t)(800.*PSC_CLOCK/1e9))
//#define PWM_DEADTIME 5
//#define PWM_DEADTIME 10
//#define PWM_DEADTIME 15
//#define PWM_DEADTIME 20
//#define PWM_DEADTIME 25
//#define PWM_DEADTIME 30
#define PWM_DEADTIME 35
//#define PWM_DEADTIME 42 // assembly 2012 viimeinen kadotus
//#define PWM_DEADTIME 50 // good

// PWM Frequency
// 31250 = 1024 cycletime
//#define PWM_FREQ 50000
//#define PWM_FREQ 40000
//#define PWM_FREQ 30000
//#define PWM_FREQ 26000
//#define PWM_FREQ 23000
#define PWM_FREQ 20000
//#define PWM_FREQ 18000 // still works
// 15000Hz will cause a short circuit

// Length of PWM cycle (~266)
#define PWM_CYCLETIME (PSC_CLOCK/PWM_FREQ)
// Non-deadtime of PWM (~256)
#define PWM_UNDEADTIME (PWM_CYCLETIME-PWM_DEADTIME*2)

// cycletime = 2 * rb + 2
// <=> rb = cycletime / 2 - 1
// RB value (corresponds to cycle time) (~132)
#define PSC_RB (PWM_CYCLETIME / 2 - 1)

/*
	The MOSFET Driver needs about 1us to change it's output state.

	With a 82k current limit resistor and 5pF of AVR input capacitance,
	the RC constant of a back-emf feedback line is 0.41us and maximum
	current is about 0.25mA.

	Also the driver bootstrap needs lowtime.
*/
//#define PSC_MIN_LOWTIME (PSC_CLOCK * 2 / 1000000)
//#define PSC_MIN_LOWTIME (PSC_CLOCK * 3 / 1000000)
//#define PSC_MIN_LOWTIME (PSC_CLOCK * 4 / 1000000)
//#define PSC_MIN_LOWTIME (PSC_CLOCK * 6 / 1000000)
//#define PSC_MIN_LOWTIME (PSC_CLOCK * 7 / 1000000) // assembly 2012 viimeinen kadotus
//#define PSC_MIN_LOWTIME (PSC_CLOCK * 8 / 1000000) // good
#define PSC_MIN_LOWTIME (PSC_CLOCK * 9 / 1000000)
//#define PSC_MIN_LOWTIME (PSC_CLOCK * 16 / 1000000)

#define PSC_MAX_HIGHTIME (PWM_CYCLETIME/2 - PWM_DEADTIME - PSC_MIN_LOWTIME)
// Limit hightime a lot, we don't need the full voltage anyway
//#define PSC_MAX_HIGHTIME (PWM_CYCLETIME/2 - PWM_DEADTIME - PSC_MIN_LOWTIME) / 2
//#define PSC_MAX_HIGHTIME (PWM_CYCLETIME/2 - PWM_DEADTIME - PSC_MIN_LOWTIME) / 10
//#define PSC_MAX_HIGHTIME (PWM_CYCLETIME/2 - PWM_DEADTIME - PSC_MIN_LOWTIME) / 5
//#define PSC_MAX_HIGHTIME (PWM_CYCLETIME/2 - PWM_DEADTIME - PSC_MIN_LOWTIME) / 3
//#define PSC_MAX_HIGHTIME (PWM_CYCLETIME/2 - PWM_DEADTIME - PSC_MIN_LOWTIME) / 4

/*
	We don't want a very low duty cycle when we have speed, because
	then the current can't be measured.
	TODO: ^
*/
//#define HIGH_SPEED_MAX_PHASE_LENGTH (TIMER1_CLOCK/(500*6/60))
//#define HIGH_SPEED_MIN_HIGHTIME (PSC_CLOCK * 15 / 10000000)

// Current is not sensed at under this
#define MIN_HIGHTIME (PSC_CLOCK * 3 / 1000000)

// PSC channel parameters: floating
#define FLOAT_PSC_SA 0
#define FLOAT_PSC_SB (PSC_RB+1)
// PSC channel parameters: pulled to ground
#define LOW_PSC_SA 0
#define LOW_PSC_SB 0

// =========================================================================
//                                                             CONSTANT DATA
// =========================================================================

const uint8_t exp128[128] = 
{
1, 1, 1, 2, 2, 3, 3, 3, 
4, 4, 5, 5, 6, 7, 7, 8, 
9, 9, 10, 11, 12, 13, 13, 14, 
15, 16, 17, 18, 19, 20, 21, 22, 
23, 25, 26, 27, 28, 30, 31, 32, 
33, 35, 36, 38, 39, 41, 42, 44, 
45, 47, 48, 50, 52, 53, 55, 57, 
59, 60, 62, 64, 66, 68, 70, 72, 
74, 76, 78, 80, 82, 84, 86, 89, 
91, 93, 95, 98, 100, 102, 105, 107, 
110, 112, 114, 117, 120, 122, 125, 127, 
130, 133, 135, 138, 141, 144, 146, 149, 
152, 155, 158, 161, 164, 167, 170, 173, 
176, 179, 182, 185, 189, 192, 195, 198, 
202, 205, 208, 212, 215, 219, 222, 226, 
229, 233, 236, 240, 243, 247, 251, 255, 
};

/*
	Defines the commutation sequence to one direction.
*/
#define COMM_TABLE_SIZE 6
struct comm_state g_comm_table[COMM_TABLE_SIZE] = 
{
	{.high=PSC_A, .low=PSC_B, .floating=PSC_C, .rising=PSC_B},
	{.high=PSC_C, .low=PSC_B, .floating=PSC_A, .rising=PSC_C},
	{.high=PSC_C, .low=PSC_A, .floating=PSC_B, .rising=PSC_B},
	{.high=PSC_B, .low=PSC_A, .floating=PSC_C, .rising=PSC_C},
	{.high=PSC_B, .low=PSC_C, .floating=PSC_A, .rising=PSC_B},
	{.high=PSC_A, .low=PSC_C, .floating=PSC_B, .rising=PSC_C},
};

// =========================================================================
//                                                          GLOBAL VARIABLES
// =========================================================================

// Control parameters for high, low and floating PSC channels.
// These are set to the corresponding high, low and floating channel.
// High and low channel parameters are modified according to PWM.
volatile struct PscControl g_psc_high_control  = {FLOAT_PSC_SA, FLOAT_PSC_SB};
volatile struct PscControl g_psc_low_control   = {FLOAT_PSC_SA, FLOAT_PSC_SB};
const    struct PscControl g_psc_float_control = {FLOAT_PSC_SA, FLOAT_PSC_SB};

// The id of the PSC channel supplying positive voltage
volatile uint8_t g_high_psc = 0;
// The id of the PSC channel connecting to ground
volatile uint8_t g_low_psc = 1;
// The id of the PSC channel that is floating
volatile uint8_t g_float_psc = 2;

// The last comparator status of the floating PSC
// This has to be cleared to 255 when the floating PSC is changed
volatile uint8_t g_float_ph_last_state = 255;
volatile uint8_t g_float_ph_last_stable_state = 255;

// This is decremented while the floating phase remains constant.
// When this is 0, floating phase is considered stable and value is sampled.
volatile uint8_t g_float_ph_stability_counter = 3;

volatile uint8_t g_comm_index = 0;
volatile uint8_t g_reverse = 0;

// Servo 2 input
volatile uint8_t timer0_ovf_counter_servo2 = 0;
volatile uint8_t servo2_state;
volatile uint8_t servo2_value;
volatile uint8_t servo2_valid = 0;
volatile uint8_t servo2_error_code = 0;

// Servo 1 input
volatile uint8_t timer0_ovf_counter_servo1 = 0;
volatile uint8_t servo1_state;
volatile uint8_t servo1_value;
volatile uint8_t servo1_valid = 0;
volatile uint8_t servo1_error_code = 0;

volatile uint8_t g_target_throttle = 0; // 0 - 127
volatile uint8_t g_target_reverse = 0;

volatile uint16_t g_comm_phase_length = UINT16_MAX;

volatile uint16_t g_runtime_ms = 0;
volatile uint16_t g_voltage = 12000;
volatile int16_t g_temperature = 20;
volatile int16_t g_accel = 0;
volatile int16_t g_accel_max_recorded = -10000;
volatile int16_t g_accel_min_recorded = 10000;

// Sensor state bitfield
volatile uint8_t g_sensor_state = 0;

// Translational drift
// angles are CCW
volatile uint16_t g_angle_robot_re_ground = 0;
volatile uint16_t g_dir_re_robot = 0;

// Watchdog
volatile uint8_t g_wdt_timer_servo2_has_input = 0;
volatile uint8_t g_wdt_timer_servo1_has_input = 0;
volatile uint8_t g_wdt_timer_sensor_state_changed = 0;

// USART
#define USART_TX_BUF_SIZE 30
volatile uint8_t g_usart_tx_buf[USART_TX_BUF_SIZE];
volatile uint8_t g_usart_tx_buf_start = 0;
volatile uint8_t g_usart_tx_buf_next = 0;
volatile uint8_t g_usart_tx_buf_size = 0;
volatile uint8_t g_usart_direct_transmit = FALSE;

// ADC read list and state
#define MAX_ADC_CHANNELS 10
volatile uint8_t g_adc_channels[MAX_ADC_CHANNELS];
volatile uint8_t g_adc_channels_count = 0;
volatile uint8_t g_adc_channel_counter = 0;

// =========================================================================
//                                             FUNCTION FORWARD DECLARATIONS
// =========================================================================

static void psc_set8(uint8_t n, struct PscControl c);

//static void usart_wait_ready(void);
static void usart_transmit(uint8_t data);
static void usart_transmit_text(const char *p);
static void usart_transmit_text_P(const char *p);
#define tx(x) usart_transmit_text(x)
#define tx_P(x) usart_transmit_text_P(x)
#define tx_crlf(x) usart_transmit_text_P(PSTR("\r\n"))

static void wdt_run_timers(void);
static uint8_t servo_input_with_wdt_ok(void);
static uint8_t wdt_everything_ok(void);

// =========================================================================
//                                                                 FUNCTIONS
// =========================================================================

/*
	Debug
*/

#define tx_num(buf, x){\
	tx_P(PSTR(#x ": "));\
	utoa(x, buf, 10);\
	tx(buf);\
	tx_crlf();\
}

#define tx_hex(buf, x){\
	tx_P(PSTR(#x ": 0x"));\
	utoa(x, buf, 16);\
	tx(buf);\
	tx_crlf();\
}

#define tx_bits(buf, x){\
	tx_P(PSTR(#x ": 0b"));\
	tx_P(x&0x80?PSTR("1"):PSTR("0"));\
	tx_P(x&0x40?PSTR("1"):PSTR("0"));\
	tx_P(x&0x20?PSTR("1"):PSTR("0"));\
	tx_P(x&0x10?PSTR("1"):PSTR("0"));\
	tx_P(x&0x08?PSTR("1"):PSTR("0"));\
	tx_P(x&0x04?PSTR("1"):PSTR("0"));\
	tx_P(x&0x02?PSTR("1"):PSTR("0"));\
	tx_P(x&0x01?PSTR("1"):PSTR("0"));\
	tx_crlf();\
}

static void transmit_info(void)
{
	char buf[12];

	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		// Reset PCSs
		{
			struct PscControl floatcontrol = {FLOAT_PSC_SA, FLOAT_PSC_SB};
			psc_set8(PSC_A, floatcontrol);
			psc_set8(PSC_B, floatcontrol);
			psc_set8(PSC_C, floatcontrol);
		}
		
		g_usart_direct_transmit = TRUE;
		
		tx_P(PSTR("INFO: \r\n"));

		tx_num(buf, g_voltage);
		tx_num(buf, g_temperature);
		tx_num(buf, g_accel);
		tx_num(buf, g_accel_max_recorded);
		tx_num(buf, -g_accel_min_recorded);
		tx_num(buf, g_runtime_ms);
		tx_num(buf, g_reverse);
		tx_num(buf, g_comm_phase_length);
		tx_num(buf, servo2_state);
		tx_num(buf, servo2_value);
		tx_num(buf, servo2_valid);
		tx_num(buf, servo2_error_code);
		tx_num(buf, g_wdt_timer_servo2_has_input);
		tx_num(buf, servo1_state);
		tx_num(buf, servo1_value);
		tx_num(buf, servo1_valid);
		tx_num(buf, servo1_error_code);
		tx_num(buf, g_wdt_timer_servo1_has_input);
		tx_bits(buf, g_sensor_state);
		if(c_comm_mode == COMM_SENSOR)
			tx_num(buf, g_wdt_timer_sensor_state_changed);
		tx_num(buf, PWM_CYCLETIME);
		tx_num(buf, PWM_DEADTIME);
		tx_num(buf, PSC_MIN_LOWTIME);
		
		tx_P(PSTR("driver fault: "));
		if(!(DRIVER_FAULT_PIN & (1<<DRIVER_FAULT_BIT)))
			tx_P(PSTR("TRUE"));
		else
			tx_P(PSTR("FALSE"));
		tx_crlf();

		tx_P(PSTR("wdt_everything_ok(): "));
		utoa(wdt_everything_ok(), buf, 10);
		tx(buf);
		tx_crlf();

		g_usart_direct_transmit = FALSE;
	}
}

static void panic(uint8_t n)
{
	uint8_t i;
	char buf[12];

	cli();
	struct PscControl floatcontrol = {FLOAT_PSC_SA, FLOAT_PSC_SB};
	psc_set8(PSC_A, floatcontrol);
	psc_set8(PSC_B, floatcontrol);
	psc_set8(PSC_C, floatcontrol);
	PCTL0 &= ~(1<<PRUN0);

	wdt_enable(WDTO_1S);

	led_off();
	_delay_ms(500);

	if(n == 0)
		n = 1;
	
	for(;;)
	{
		// Send stuff to serial
		g_usart_direct_transmit = TRUE;
		utoa(n, buf, 10);
		tx_P(PSTR("PANIC "));
		tx(buf);
		tx_crlf();
		g_usart_direct_transmit = FALSE;

		transmit_info();

		// Blink LED
		for(i=0; i<n; i++){
			wdt_reset();
			led_on();
			_delay_ms(250);
			if(button_pressed()) goto panic_end;
			wdt_reset();
			led_off();
			_delay_ms(250);
			if(button_pressed()) goto panic_end;
		}
		wdt_reset();
		_delay_ms(500);
		if(button_pressed()) goto panic_end;
		wdt_reset();
		_delay_ms(500);
		if(button_pressed()) goto panic_end;
	}
	
panic_end:
	// Perform a watchdog timeout
	for(;;){
		led_on();
		_delay_ms(100);
		led_off();
		_delay_ms(100);
	}
}

/* Integer square root by Halleck's method, with Legalize's speedup */
uint32_t isqrt (uint32_t x)
{
  uint32_t   squaredbit, remainder, root;

   if (x<1) return 0;
  
   /* Load the binary constant 01 00 00 ... 00, where the number
    * of zero bits to the right of the single one bit
    * is even, and the one bit is as far left as is consistant
    * with that condition.)
    */
   squaredbit  = (uint32_t) ((((uint32_t) ~0L) >> 1) & 
                        ~(((uint32_t) ~0L) >> 2));
   /* This portable load replaces the loop that used to be 
    * here, and was donated by  legalize@xmission.com 
    */

   /* Form bits of the answer. */
   remainder = x;  root = 0;
   while (squaredbit > 0) {
     if (remainder >= (squaredbit | root)) {
         remainder -= (squaredbit | root);
         root >>= 1; root |= squaredbit;
     } else {
         root >>= 1;
     }
     squaredbit >>= 2; 
   }

   return root;
}

/*
	Commutation sensor
*/

static void update_sensor_state(void)
{
	uint8_t newstate = 0;
	uint8_t highcount = 0;
	if(S1_PORT_IN & (1<<S1_BIT))
	{
		newstate |= (1<<0);
		highcount++;
	}
	if(S2_PORT_IN & (1<<S2_BIT))
	{
		newstate |= (1<<1);
		highcount++;
	}
	if(S3_PORT_IN & (1<<S3_BIT))
	{
		newstate |= (1<<2);
		highcount++;
	}
	// Update global state if state is valid
	if(highcount == 1 || highcount == 2)
	{
		g_sensor_state = newstate;
	}
}

/*
	PSC
*/

// Lock PSCs to not be modifiable and apply set parameters
static void psc_lock(void)
{
	PCNF0 = (1<<PLOCK0) | PCNF_SETTING;
	PCNF1 = (1<<PLOCK1) | PCNF_SETTING;
	PCNF2 = (1<<PLOCK2) | PCNF_SETTING;
}

// Unlock PSCs for parameter change
static void psc_unlock(void)
{
	PCNF0 = PCNF_SETTING;
	PCNF1 = PCNF_SETTING;
	PCNF2 = PCNF_SETTING;
}

/*
	Set PSC channel parameters.
	To change nothing, set n=PSC_NONE.
*/
static void psc_set8(uint8_t n, struct PscControl c)
{
	switch(n)
	{
	case 0:
		OCR0SB = c.sb;
		OCR0SA = c.sa;
		break;
	case 1:
		OCR1SB = c.sb;
		OCR1SA = c.sa;
		break;
	case 2:
		OCR2SB = c.sb;
		OCR2SA = c.sa;
		break;
	}
}

static void psc_apply(void)
{
	//DACH = g_comm_index * 40;
	psc_lock();
	if(c_comm_mode == COMM_SENSOR)
	{
		if(g_reverse == FALSE)
		{
			if(g_sensor_state & (1<<0))
				psc_set8(PSC_A, g_psc_high_control);
			else
				psc_set8(PSC_A, g_psc_low_control);

			if(g_sensor_state & (1<<1))
				psc_set8(PSC_C, g_psc_high_control);
			else
				psc_set8(PSC_C, g_psc_low_control);

			if(g_sensor_state & (1<<2))
				psc_set8(PSC_B, g_psc_high_control);
			else
				psc_set8(PSC_B, g_psc_low_control);
		}
		else
		{
			if(g_sensor_state & (1<<0))
				psc_set8(PSC_A, g_psc_low_control);
			else
				psc_set8(PSC_A, g_psc_high_control);

			if(g_sensor_state & (1<<1))
				psc_set8(PSC_C, g_psc_low_control);
			else
				psc_set8(PSC_C, g_psc_high_control);

			if(g_sensor_state & (1<<2))
				psc_set8(PSC_B, g_psc_low_control);
			else
				psc_set8(PSC_B, g_psc_high_control);
		}
	}
	else if(c_comm_mode == COMM_MAINLOOP || c_comm_mode == COMM_INTERRUPT)
	{
		psc_set8(g_high_psc,  g_psc_high_control);
		psc_set8(g_low_psc,   g_psc_low_control);
		psc_set8(g_float_psc, g_psc_float_control);
	}
	else if(c_comm_mode == COMM_NONE)
	{
		struct PscControl floatcontrol = {FLOAT_PSC_SA, FLOAT_PSC_SB};
		psc_set8(PSC_A, floatcontrol);
		psc_set8(PSC_B, floatcontrol);
		psc_set8(PSC_C, floatcontrol);
	}
	psc_unlock();
}

#define HIGHTIME_TO_SA(x) (x>PSC_MAX_HIGHTIME ? PSC_MAX_HIGHTIME:x)
#define HIGHTIME_TO_SB(x) (x>PSC_MAX_HIGHTIME ? (PSC_MAX_HIGHTIME+PWM_DEADTIME):(x+PWM_DEADTIME))

static void psc_drive(uint8_t hightime)
{
	g_psc_high_control.sa = HIGHTIME_TO_SA(hightime);
	g_psc_high_control.sb = HIGHTIME_TO_SB(hightime);
	g_psc_low_control.sa = LOW_PSC_SA;
	g_psc_low_control.sb = LOW_PSC_SB;
}

static void psc_freewheel(void)
{
	g_psc_high_control.sa = FLOAT_PSC_SA;
	g_psc_high_control.sb = FLOAT_PSC_SB;
	g_psc_low_control.sa = FLOAT_PSC_SA;
	g_psc_low_control.sb = FLOAT_PSC_SB;
}

/*
	Set n=PSC_NONE to set nothing.
	sa Controls PSCOUTn0 (High Side) ON-time
	sb Controls PSCOUTn1 (Low Side) OFF-time
*/
/*static void psc_set(uint8_t n, uint16_t sa, uint16_t sb)
{
	// Set values
	switch(n)
	{
	case 0:
		OCR0SB = sb;
		OCR0SA = sa;
		break;
	case 1:
		OCR1SB = sb;
		OCR1SA = sa;
		break;
	case 2:
		OCR2SB = sb;
		OCR2SA = sa;
		break;
	//default:
	//	panic(10);
	}
}*/

/*
	Set n=PSC_NONE to disable nothing.
*/
/*static void psc_float(uint8_t n)
{
	psc_set(n, 0, PSC_RB+1);
}*/

/*
	Set n=PSC_NONE to enable nothing.
*/
/*static void psc_enable(uint8_t n, uint16_t hightime)
{
	// Controls PSCOUTn0 (High Side) On-time
	uint16_t sa = hightime / 2;
	// Controls PSCOUTn1 (Low Side) OFF-time
	uint16_t sb = PWM_DEADTIME + hightime / 2;

	psc_set(n, sa, sb);
}*/

static void init_psc(void)
{
	// We're configuring most of the stuff (if not all) identically for all three.
	{
		uint8_t b = 0;
		// PSC n Part B Output Enable
		b |= (1<<POEN0B);
		// PSC n Part A Output Enable
		b |= (1<<POEN0A);
		// PSYNCn1:0: Synchronization Out for ADC Selection
		// Amplifier does sampling according to this
		// Syncronize ADC at OCRnRA when counting up
		b &= ~(1<<PSYNC01);
		b |=  (1<<PSYNC00);
		PSOC0 = b;
		PSOC1 = b;
		PSOC2 = b;
	}
	{
		PCNF0 = PCNF_SETTING;
		PCNF1 = PCNF_SETTING;
		PCNF2 = PCNF_SETTING;
	}
	{
		uint8_t b = 0;
		// PSC 0 Capture Enable Input Part A:
		// Disable
		b &= ~(1<<PCAE0A);
		// PSC 0 Input Select for Part A:
		// Comparator
		b |= (1<<PISEL0A);
		PFRC0A = b;
		PFRC1A = b;
		PFRC2A = b;
		PFRC0B = b;
		PFRC1B = b;
		PFRC2B = b;
	}
	{
		uint8_t b = 0;
		// PSC n Synchro Error Interrupt Enable:
		// Disable
		b &= ~(1<<PSEIE0);
		// PSC n External Event B Interrupt Enable:
		// Disable
		b &= ~(1<<PEVE0B);
		// PSC n External Event A Interrupt Enable:
		// Disable
		b &= ~(1<<PEVE0A);
		// PSC n End Of Cycle Interrupt Enable:
		// Disable
		b &= ~(1<<PEOPE0);
		PIM0 = b;
		PIM1 = b;
		PIM2 = b;
	}
	{
		// OCRnRB - Controls cycle time in Center Aligned Mode
		OCR0RB = PSC_RB;
		OCR1RB = PSC_RB;
		OCR2RB = PSC_RB;
		// OCRnRA - Controls amp sampling in Center Aligned Mode
		OCR0RA = 1;
		//OCR1RA = 1;
		//OCR2RA = 1;
		psc_apply(); //apply default values
		/*psc_float(0);
		psc_float(1);
		psc_float(2);*/
	}
	{
		// PSC 0 End Of Cycle Interrupt Enable:
		// Enable
		PIM0 |= (1<<PEOPE0);
	}
	{
		uint8_t b = 0;
		// PSC 0 Prescaler select:
		// PPRE01 PPRE00 Description
		// 0      0      No divider
		// 0      1      Divide PCB input clock by 4
		// 1      0      Divide by 32 (at90pwm2/3: divide by 16)
		// 1      1      Divide by 256 (at90pwm2/3: divide by 64)
		//b |= (0<<PPRE01) | (0<<PPRE00); // divider=1
		b |= (0<<PPRE01) | (1<<PPRE00); // divider=4
		// Set 1 and 2 to be autorun when 0 starts.
		PCTL1 = b | (1<<PARUN1);
		PCTL2 = b | (1<<PARUN2);
		// Start 0
		PCTL0 = b | (1<<PRUN0);
		/*PCTL0 = b | (1<<PRUN0);
		PCTL1 = b | (1<<PRUN1);
		PCTL2 = b | (1<<PRUN2);*/
	}
	/*{
		uint8_t b = 0;
		 = b;
	}*/
}

/*inline uint16_t limit_lowtime(uint16_t lowtime)
{
	if(lowtime < PWM_MINIMUM_LOWTIME)
		return PWM_MINIMUM_LOWTIME;
	else if(lowtime > PSC_RB + 1)
		return PSC_RB + 1;
	return lowtime;
}

inline uint16_t get_hightime(uint16_t lowtime)
{
	if(lowtime < PSC_RB + 1 - PWM_DEADTIME*2)
		return PSC_RB + 1 - PWM_DEADTIME*2 - lowtime;
	else
		return 0;
}*/

/*static void psc_drive(uint16_t lowtime)
{
	lowtime = limit_lowtime(lowtime);
	uint16_t hightime = get_hightime(lowtime);
	//sa Controls PSCOUTn0 (High Side) ON-time
	//sb Controls PSCOUTn1 (Low Side) OFF-time
	g_low_psc_sa = 0;
	g_low_psc_sb = 0;
	g_high_psc_sa = hightime;
	g_high_psc_sb = PSC_RB + 1 - lowtime;
	//g_high_psc_sb = PSC_RB + 1 - PWM_MINIMUM_LOWTIME;
}*/

/*
	Select which PSC is high, low and floating.
	
	This is called when the commutation index changes.
	After this, psc_apply() has to be called.
	
	FIXME: g_floating_changed_delay_counter and _last_state should be resetted in psc_apply().
	       This is not a problem if psc_apply() is called immediately after psc_select().
*/
static void psc_select(uint8_t high_psc, uint8_t low_psc, uint8_t floating_psc)
{
	g_high_psc = high_psc;
	g_low_psc = low_psc;
	//if(floating_psc != g_floating_psc){
	g_float_psc = floating_psc;
	g_float_ph_last_state = 255;
	g_float_ph_last_stable_state = 255;
	//}
}

/*
	Servo input
*/

static void servo2_start_input(void)
{
	// Rising edge on INT1 generates an interrupt request
	EICRA |= (1<<ISC11) | (1<<ISC10);
	// Enable INT1
	EIMSK |= (1<<INT1);
	// Clear interrupt
	EIFR |= (1<<INTF1);
	// Low state
	servo2_state = 0;
	timer0_ovf_counter_servo2 = 0;
}

static void servo1_start_input(void)
{
	// Rising edge on INT2 generates an interrupt request
	EICRA |= (1<<ISC21) | (1<<ISC20);
	// Enable INT1
	EIMSK |= (1<<INT2);
	// Clear interrupt
	EIFR |= (1<<INTF2);
	// Low state
	servo1_state = 0;
	timer0_ovf_counter_servo2 = 0;
}

static void servo2_stop_input(void)
{
	// Disable INT1
	EIMSK &= ~(1<<INT1);
	// Clear interrupt
	EIFR |= (1<<INTF1);
	// Inactive state
	servo2_state = 2;
}

static void servo1_stop_input(void)
{
	// Disable INT2
	EIMSK &= ~(1<<INT2);
	// Clear interrupt
	EIFR |= (1<<INTF2);
	// Inactive state
	servo1_state = 2;
}

static void servo_init(void)
{
	// Only start one
	if(ENABLE_SERVO2_INPUT)
		servo2_start_input();
	else if(ENABLE_SERVO1_INPUT)
		servo1_start_input();
}

static uint8_t servo_input_ok(void)
{
	if(ENABLE_SERVO2_INPUT && servo2_valid == FALSE)
		return FALSE;
	if(ENABLE_SERVO1_INPUT && servo1_valid == FALSE)
		return FALSE;
	return TRUE;
}

/*static uint8_t servo_is_nonzero(uint8_t val)
{
	return (val < 128-5 || val > 128+5);
}*/

static void servo2_report_invalid(uint8_t error_code)
{
	servo2_valid = FALSE;
	servo2_error_code = error_code;
}

static void servo2_report_valid(uint8_t value)
{
	// Average a little if the last value was valid
	if(servo2_valid){
		servo2_value = servo2_value/2 + value/2;
	}
	else{
		servo2_value = value;
	}
	servo2_valid = TRUE;
	servo2_error_code = 0;

	// Use the value

	int16_t a = 128 - servo2_value;

	if(c_throttle_mode == THROTTLE_ORIGINAL)
	{
		static uint8_t throttle_on = 0;
		if(a < 0){
			a = -a;
			if(a > 50){
				a = 0;
			}
			g_reverse = 1;
		}
		else{
			g_reverse = 0;
		}
		if(a < 10 - throttle_on*2){
			a = 0;
			throttle_on = 0;
		}
		else{
			throttle_on = 1;
		}
		//a *= PSC_RB/128;
		a += a / 2; //adjust value to reach all the way up
		if(a > PSC_MAX_HIGHTIME)
			a = PSC_MAX_HIGHTIME;

		psc_drive(a);
		psc_apply();

		if(a != 0)
			g_wdt_timer_servo2_has_input = 255;
	}
	else if(c_throttle_mode == THROTTLE_NEW)
	{
		int16_t deadzone = RC_DEADZONE;
		if(a < 0){
			a = -a;
			/*if(a > 50){
				a = 0;
			}*/
			g_target_reverse = 1;
		}
		else{
			if(a <= deadzone)
				a = 0;
			else
				a -= deadzone;
			g_target_reverse = 0;
		}
		a += a/2 + a/3; // adjust value to reach all the way up
		if(a > 127)
			a = 127;
		if(EXP_THROTTLE)
			a = exp128[a]/2;
		g_target_throttle = a;

		if(a != 0)
			g_wdt_timer_servo2_has_input = 255;
	}
}

static void servo1_report_invalid(uint8_t error_code)
{
	servo1_valid = FALSE;
	servo1_error_code = error_code;
}

static void servo1_report_valid(uint8_t value)
{
	// Average a little if the last value was valid
	if(servo1_valid){
		servo1_value = servo1_value/2 + value/2;
	}
	else{
		servo1_value = value;
	}
	servo1_valid = TRUE;
	servo1_error_code = 0;

	g_wdt_timer_servo1_has_input = 255;

	// Use the value
	
	if(ENABLE_MELTY)
	{
		g_dir_re_robot += ((int16_t)servo1_value - 128) * 32;
	}
}

/*
	USART
*/

#define d_sendtext(x) usart_transmit_text_P(PSTR(x))
#define d_sendvar(x, name){\
	char buf[12];\
	ltoa(x, buf, 10);\
	d_sendtext("[" name "=");\
	usart_transmit_text(buf);\
	d_sendtext("]");\
}

// baud in Hz
static void usart_init(uint16_t baud)
{
	// Set baud rate
	UBRR = F_CPU / 8 / baud - 1;
	// Baud rate divider 8
	UCSRA = (1<<U2X);
	// Set frame format: 8 data, no parity, 2 stop bits
	UCSRC = (0<<UMSEL0)|(0<<UPM0)|(0<<USBS)|(1<<UCSZ1)|(1<<UCSZ0);
	// Enable receiver and transmitter
	//UCSRB = (1<<RXEN)|(1<<TXEN);
	// Enable receiver and transmitter and RX interrupt
	//UCSRB = (1<<RXEN)|(1<<TXEN)|(1<<RXCIE);
	// Enable transmitter and RX interrupt
	//UCSRB = (1<<TXEN)|(1<<RXCIE);
	// Enable transmitter and receiver and RX interrupt and TX interrupt
	UCSRB = (1<<TXEN)|(1<<RXEN)|(1<<RXCIE)|(1<<TXCIE);
}

/*static void usart_rx_disable(void)
{
	UCSRB &= ~(1<<RXEN);
}

static void usart_rx_enable(void)
{
	UCSRB |= (1<<RXEN);
}*/

#define USART_INDEX_INCREMENT(x){\
	if(x < USART_TX_BUF_SIZE - 1)\
		x++;\
	else\
		x = 0;\
}

/*static void usart_wait_ready(void)
{
	while(!(UCSRA & (1<<UDRE)));
}*/

// Transmit a byte.
// Byte is buffered if USART is not ready or there is stuff in the buffer.
static void usart_transmit(uint8_t data)
{
	// If direct transmit is enabled, don't buffer anything
	if(g_usart_direct_transmit == TRUE)
	{
		while(!(UCSRA & (1<<UDRE)));
		UDR = data;
		return;
	}

	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		// If USART is ready and buffer is empty, just send it
		if((UCSRA & (1<<UDRE)) && g_usart_tx_buf_size == 0)
		{
			UDR = data;
		}
		// Else buffer it.
		else
		{
			// ...Buffer only if there is space in the buffer
			if(g_usart_tx_buf_size < USART_TX_BUF_SIZE)
			{
				g_usart_tx_buf[g_usart_tx_buf_next] = data;
				USART_INDEX_INCREMENT(g_usart_tx_buf_next);
				g_usart_tx_buf_size++;
			}
		}
	}
}

static void usart_transmit_text(const char *p)
{
	if(p==NULL) return;
	if(p[0]==0) return;
	do{
		usart_transmit(*p);
	}while(*(++p));
}

static void usart_transmit_text_P(const char *p)
{
	if(p==NULL) return;
	if(pgm_read_byte(&p[0])==0) return;
	do{
		usart_transmit(pgm_read_byte(p));
	}while(pgm_read_byte(++p));
}

/*
	Timers
*/

static void init_timer(void)
{
	/*
		TIMER1
		This timer is completely in use by commutation code.
	*/
	{
		// TCCR1A
		uint8_t b = 0;
		// This register provides only output (PWM) options.
		TCCR1A = b;
	}
	{
		// TCCR1B
		uint8_t b = 0;
		// Clock Select:
		// clk_io / 64 -> 250kHz = 4us/1bit = 262144us/16bit = 262ms = 3.8Hz
		b &= ~(1<<CS12);
		b |= (1<<CS11);
		b |= (1<<CS10);
#define TIMER1_CLOCK (F_CPU/64) // 16M/64=250k
		/*// clk_io / 8 -> 2MHz = 0.5us/bit = 32768us/16bit = 33ms = 30Hz
		b &= ~(1<<CS12);
		b |=  (1<<CS11);
		b &= ~(1<<CS10);
#define TIMER1_CLOCK (F_CPU/8)*/
		// Consult the datasheet for other options.
		TCCR1B = b;
	}
	/*
		TIMER0
	*/
	{
		// TCCR0A
		uint8_t b = 0;
		// This register provides only output (PWM) options.
		TCCR0A = b;
	}
	{
		// TCCR0B
		uint8_t b = 0;
		// Clock Select:
		/*
		// clk_io/256
		b |= (1<<CS02);
		b &= ~(1<<CS01);
		b &= ~(1<<CS00);
#define TIMER0_CLOCK (F_CPU/256)
		*/
		// clk_io/64
		b &= ~(1<<CS02);
		b |=  (1<<CS01);
		b |=  (1<<CS00);
#define TIMER0_CLOCK (F_CPU/64)
		/*
		// clk_io/8
		b &= ~(1<<CS02);
		b |=  (1<<CS01);
		b &= ~(1<<CS00);
#define TIMER0_CLOCK (F_CPU/8)
*/
		// Consult the datasheet for other options.
		TCCR0B = b;
	}
	{
		// OCR0A - Timer 0 Output compare register A
/*#define TIMER0_A_FREQ 10000
		OCR0A = TIMER0_CLOCK / TIMER0_A_FREQ;*/
		OCR0A = 0;
	}
	{
		// TIMSK0
		uint8_t b = 0;
		// match A interrupt enable
		b |= (1<<OCIE0A);
		// overflow interrupt enable
		b |= (1<<TOIE0);
		TIMSK0 = b;
	}
}

/*
	Commutation
*/

static uint8_t get_next_comm_index(uint8_t current)
{
	if(current < COMM_TABLE_SIZE-1)
		return current + 1;
	else
		return 0;
}

static uint8_t get_next_comm_index_reverse(uint8_t current)
{
	if(current > 0)
		return current - 1;
	else
		return COMM_TABLE_SIZE-1;
}

static void psc_select_from_table(void)
{
	uint8_t i = g_comm_index;
	psc_select(g_comm_table[i].high, g_comm_table[i].low, g_comm_table[i].floating);
}

static void handle_adc_current(void)
{
	int8_t current = ADCH;
	//DACH = current + 128;
	if(c_throttle_mode == THROTTLE_NEW)
	{
		static uint8_t drive = 0;

		if(g_voltage < 8000 || g_temperature > 100)
		{
			drive = 0;
			psc_freewheel();
			psc_apply();
			return;
		}
		
		//drive = g_target_throttle;
		
		/*
			This has to be changed for different PWM frequencies.
			20kHz: g_target_throttle
			40kHz: g_target_throttle / 2
		*/
		// 40kHz:
		//uint8_t throttle = g_target_throttle / 2;
		// 30kHz:
		uint8_t throttle = g_target_throttle - g_target_throttle / 3;
		// 20kHz:
		//uint8_t throttle = g_target_throttle;

		const int8_t max_current = MAXIMUM_CURRENT;

		uint8_t changing_direction = (g_reverse != g_target_reverse);

		int8_t max_negative_current;
		
		if(c_brake_mode == BRAKE_ROBOT)
		{
			//max_negative_current = -30;
			max_negative_current = -(max_current - max_current/3);
			static uint8_t changedircount = 0;
			if(changing_direction){
				// Throw throttle to zero
				throttle = 0;
				// Delay a bit to measure actual current with throttle at 0.
				changedircount++;
				if(changedircount >= 4){
					// When the current has lowered enough, switch to reverse
					if(current > -20){
						g_reverse = g_target_reverse;
					}
				}
			}
			else
				changedircount = 0;
		}
		else if(c_brake_mode == BRAKE_CAR)
		{
			// A small value that is still detectable
			max_negative_current = -3;
			if(changing_direction){
				max_negative_current -= g_target_throttle/4;
				throttle = 0;
				if(g_comm_phase_length > TIMER1_CLOCK/(5000/60)/6){
					g_reverse = g_target_reverse;
				}
			}
		}
		
		/*
			A special hack for a receiver that sets throttle to full reverse
			when signal is not received
		*/
		if(DISABLE_FAST_REVERSE && g_target_reverse && g_target_throttle > 90)
		{
			throttle = 0;
		}
		
		if(current > max_current && drive >= MIN_HIGHTIME)
		{
			if(drive > 0)
				drive--;
		}
		/*if(current > 45 && g_comm_phase_length > TIMER1_CLOCK/(5000/60)/6)
		{
			if(drive > 0)
				drive--;
		}
		else if(current > 80)
		{
			if(drive > 0)
				drive--;
		}*/
		else if(current < max_negative_current)
		{
			if(drive < PSC_MAX_HIGHTIME)
				drive++;
		}
		else
		{
			//if(g_target_throttle + g_target_throttle/2 > drive){
			/*
				Allow faster throttle changing per call at low speeds
			*/
			const uint8_t D = 4;
			const uint8_t D2 = 2;
			// Limit speed
			if(g_comm_phase_length < TIMER1_CLOCK/(MAX_MOTOR_RPM/60)/6)
			{
				if(drive > 2)
					drive--;
			}
			else if(throttle > drive + D
					/*&& drive < PSC_MAX_HIGHTIME - D*/
					&& g_comm_phase_length > TIMER1_CLOCK/21){
				drive += D;
			}
			else if(throttle < drive - D
					/*&& drive >= D*/
					&& g_comm_phase_length > TIMER1_CLOCK/21){
				drive -= D;
			}
			else if(throttle > drive + D2){
				if(drive < PSC_MAX_HIGHTIME - D2)
					drive += D2;
			}
			else if(throttle < drive - D2){
				if(drive > D2)
					drive-=D2;
			}
			else if(throttle > drive){
				if(drive < PSC_MAX_HIGHTIME)
					drive++;
			}
			else{
				if(drive > 0)
					drive--;
			}
		}

		uint8_t dr = drive;
		
		if(ENABLE_MELTY)
		{
			/*if(g_comm_phase_length < TIMER1_CLOCK /
					((MAX_MOTOR_RPM - MAX_MOTOR_RPM/4)/60)/6)*/
			// servo2 goes down to 25
			if(servo2_value < 35)
			{
				uint16_t f = (35 - servo2_value)*5;
				if(f > 100)
					f = 100;
				// Drive difference
				uint16_t df = (uint16_t)drive * f / 100;
				// Double-check
				if(df > drive / 2)
					df = drive / 2;
				//uint8_t df = drive / 2;
				// Adjust timing
				uint16_t dir = g_dir_re_robot + 65535/3 + 65535/2;
				if(dir < 65535/2)
				{
					dr = drive - (uint8_t)df;
				}
				else
				{
					dr = drive + (uint8_t)df;
					if(dr > PSC_MAX_HIGHTIME)
						dr = PSC_MAX_HIGHTIME;
				}
			}
		}

		psc_drive(dr);
		psc_apply();
	}
}

static void handle_adc_servo1(void)
{
	if(CONTROL_MODE == CONTROL_SERVO1_ANALOG)
	{
		uint8_t a_in = ADCH;
		int16_t a = a_in - 120;
		
		int16_t deadzone = 10;
		if(a < 0){
			a = -a;
			/*if(a > 50){
				a = 0;
			}*/
			g_target_reverse = 1;
		}
		else{
			if(a <= deadzone)
				a = 0;
			else
				a -= deadzone;
			g_target_reverse = 0;
		}
		//a -= a/2;
		if(a > 127)
			a = 127;

		g_target_throttle = a;
	}
}

static void handle_adc_voltage(void)
{
	uint16_t voltage = ADCH;
	// 82k:12k voltage divider
	voltage *= (uint16_t)(1000.0*5.0*(82.0+12.0)/12.0/256.0);
	g_voltage = (g_voltage-g_voltage/32) + (voltage/32);
}

static void handle_adc_temperature(void)
{
	int16_t t = ADCH;
	t -= 24; // offset to 0 at room temperature
	t *= 10; // now scale is in celsius
	t = -t; // reverse
	t += 28; // offset back to room temperature
	// Average a bit
	g_temperature = (g_temperature*31 + t) / 32;
}

static void handle_adc_s2(void)
{
	int8_t a = ADCH;
	a -= 141; // offset to 0
	g_accel = (g_accel * 15) / 16 - a; // Average and change sign
	if(g_runtime_ms > 200){
		if(g_accel > g_accel_max_recorded)
			g_accel_max_recorded = g_accel;
		if(g_accel < g_accel_min_recorded)
			g_accel_min_recorded = g_accel;
	}
}

static void fast_busytask(void)
{
	/*
		If ADC is ready, handle the result and start a new
		conversion.

		This is handled here because there is free time here
		while the motor rotates towards the phase change.
	*/
	if(ADCSRA & (1<<ADIF))
	{
		if(g_adc_channels[g_adc_channel_counter] == CH_CURRENT)
		{
			handle_adc_current();
		}
		else if(g_adc_channels[g_adc_channel_counter] == CH_SERVO1)
		{
			handle_adc_servo1();
		}
		else if(g_adc_channels[g_adc_channel_counter] == CH_VOLTAGE)
		{
			handle_adc_voltage();
		}
		else if(g_adc_channels[g_adc_channel_counter] == CH_TEMPERATURE)
		{
			handle_adc_temperature();
		}
		else if(g_adc_channels[g_adc_channel_counter] == CH_S2)
		{
			handle_adc_s2();
		}

		if(g_adc_channel_counter >= g_adc_channels_count - 1)
			g_adc_channel_counter = 0;
		else
			g_adc_channel_counter++;
		ADMUX = ADMUX_SETTINGS | g_adc_channels[g_adc_channel_counter];
		ADCSRA = ADCSRA_SETTINGS | (1<<ADSC) | (1<<ADIF);
	}
}

// Called when motor has turned 1/6th rotation
static void report_rotation_step(void)
{
	// No acceleration sensor
	if(CONTROL_MODE == CONTROL_SERVO_MELTY)
	{
		uint16_t angle_add = 0;
		angle_add += (uint16_t)(WHEEL_RPM_TO_ROBOT_RPM / 6.0 * 65536.0);
		g_angle_robot_re_ground += angle_add;
		g_dir_re_robot -= angle_add;
	}
}

static void mainloop_bemf(void)
{
	/*
		Handle commutation
	*/
	if(c_comm_mode == COMM_MAINLOOP)
	{
		// Debug output
		if(c_ledmode == LEDMODE_COMM0)
		{
			if(g_comm_index == 0)
				led_on();
		}

		/*
			The previous phase has just ended and a new phase has
			been selected (at the end of the previous loop).
		*/
		
		/*
			Read phase length from timer and reset timer
		*/

		// If overflow, length is max
		if(TIFR1 & (1<<TOV1))
			g_comm_phase_length = UINT16_MAX;
		// Else read timer value
		else
			g_comm_phase_length = TCNT1;
		// Reset counter
		TCNT1 = 0;
		TIFR1 |= (1<<TOV1);
		
		/*
			Update phase to power stage control
			
			This propagates the precalculated pwm values to the
			right outputs
		*/
		
		psc_select_from_table();
		cli();
		psc_apply();
		sei();
	}
	
	fast_busytask();
	
	if(c_comm_mode == COMM_MAINLOOP)
	{
		/*
			Now the phase change has to be detected. It is done
			by busy-waiting for a global flag.
		*/
		
		/*
			Determine whether a rising or a falling edge is to
			be detected on the floating phase.
		*/
		uint8_t rising;
		if(g_reverse)
			rising = !g_comm_table[g_comm_index].rising;
		else
			rising = g_comm_table[g_comm_index].rising;
		
		// Length of timeout
		//const uint16_t T = (TIMER1_CLOCK/10); // Very slow response
		//const uint16_t T = (TIMER1_CLOCK/20);
		const uint16_t T = (TIMER1_CLOCK/30); // A bit too slow response
		//const uint16_t T = (TIMER1_CLOCK/(20000/60)/6); //umm?
		//const uint16_t T = (TIMER1_CLOCK/60); // Responsive enough
		
		/*
			At low speed the initial state of the floating phase
			will be waited for before the final state.

			It is now always true, because it makes the operation
			much more stable (almost no lock-up situations)
		*/
		// 2000rpm is pretty near the lower limit of duration (it's 44999)
		//uint8_t lowspeed = g_comm_phase_length < TIMER1_CLOCK/(2000/60)/6;
		//uint8_t lowspeed = g_comm_phase_length < TIMER1_CLOCK/5;
		uint8_t lowspeed = 1;

		if(rising){
			// Wait for initial state
			if(lowspeed)
				while(g_float_ph_last_stable_state != 0 && TCNT1 < T)
					fast_busytask();
			// Wait for final state
			while(g_float_ph_last_stable_state != 1 && TCNT1 < T)
				fast_busytask();
		}
		else{
			// Wait for initial state
			if(lowspeed)
				while(g_float_ph_last_stable_state != 1 && TCNT1 < T)
					fast_busytask();
			// Wait for final state
			while(g_float_ph_last_stable_state != 0 && TCNT1 < T)
				fast_busytask();
		}
		
		// Debug output
		if(c_ledmode == LEDMODE_COMM0)
		{
			if(g_comm_index == 0)
				led_off();
		}
		
		/*
			Increment phase counter
		*/
		if(g_reverse){
			g_comm_index = get_next_comm_index_reverse(g_comm_index);
		}
		else{
			g_comm_index = get_next_comm_index(g_comm_index);
		}
		
		// Rotated 1/6th turn
		report_rotation_step();

		//DACH = g_comm_index * 50;
	}
}

static void mainloop_sensor(void)
{
	uint8_t sensor_state_old = g_sensor_state;

	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		update_sensor_state();
		psc_apply();
	}
	
	fast_busytask();
	
	/*
		Check if phase has changed and update phase length if so
	*/
	// If overflow, length is max
	if(TIFR1 & (1<<TOV1))
	{
		// Value is maximum
		g_comm_phase_length = UINT16_MAX;
		// Reset counter
		TCNT1 = 0;
		TIFR1 |= (1<<TOV1);
	}
	else if(g_sensor_state != sensor_state_old)
	{
		g_wdt_timer_sensor_state_changed = 255;
		sensor_state_old = g_sensor_state;
		// Read timer value
		g_comm_phase_length = TCNT1;
		// Reset counter
		TCNT1 = 0;
		TIFR1 |= (1<<TOV1);

		// Rotated 1/6th turn
		report_rotation_step();
	}
}

/*
	Watchdog
*/

#define decr_timer(x){ if(x > 0) x--; }
/*
	This is run in the 1kHz interrupt
*/
static void wdt_run_timers(void)
{
	decr_timer(g_wdt_timer_servo2_has_input);
	decr_timer(g_wdt_timer_servo1_has_input);
	decr_timer(g_wdt_timer_sensor_state_changed);
}

static uint8_t servo_input_with_wdt_ok(void)
{
	if(ENABLE_SERVO2_INPUT)
	{
		/*if(g_wdt_timer_servo2_has_input == 0)
			return FALSE;*/
		if(servo2_valid == FALSE)
			return FALSE;
	}

	if(ENABLE_SERVO1_INPUT)
	{
		if(g_wdt_timer_servo1_has_input == 0)
			return FALSE;
		if(servo1_valid == FALSE)
			return FALSE;
	}

	return TRUE;
}

static uint8_t wdt_everything_ok(void)
{
	if(!servo_input_with_wdt_ok())
		return FALSE;

	if(c_comm_mode == COMM_SENSOR)
	{
		if(g_wdt_timer_sensor_state_changed == 0)
			return FALSE;
	}
	
	// If insanely high speed (speed detection fail)
	if(g_comm_phase_length < 100)
		return FALSE;
	
	// If driver reports fault, is not ok
	if(!(DRIVER_FAULT_PIN & (1<<DRIVER_FAULT_BIT)))
		return FALSE;

	return TRUE;
}

// =========================================================================
//                                                                      MAIN
// =========================================================================

uint8_t g_mcusr_mirror __attribute__ ((section (".noinit")));
void get_mcusr(void) \
	__attribute__((naked)) \
	__attribute__((section(".init3")));
void get_mcusr(void)
{
	g_mcusr_mirror = MCUSR;
	MCUSR = 0;
	wdt_disable();
}

int main(void)
{
	//char buf[12];

	g_runtime_ms = 0;
	
	wdt_enable(WDTO_1S);

	init_io();
	init_timer();
	init_analog();
	init_comparator();
	init_adc();
	init_psc();
	usart_init(RS232_BAUD);
	
	// Enable DAC (DEBUG)
	//DACON = (1<<DALA)|(1<<DAOE)|(1<<DAEN);
	//DACH = 255;
	
	// In Tuomiopäivä 666 there is a LED connected to the DAC pin PC7.
	DDRC |= (1<<7);
	PORTC |= (1<<7);
	
	// Transmit boot messages directly
	g_usart_direct_transmit = TRUE;
		
	{
		// This will come out garbled
		tx_P(PSTR(" "));
		
		//if a watchdog timer reset occurred
		if(g_mcusr_mirror & (1<<WDRF)){
			tx_P(PSTR("(watchdog) "));
		}
		else
		{
			tx_P(PSTR("\r\ndcdrive2 on tuomiopaiva 666\r\n"));
			tx_P(PSTR("reset source: "));
			//if a brown-out reset occurred
			if(g_mcusr_mirror & (1<<BORF)){
				tx_P(PSTR("brown-out"));
			}
			//if an external reset occurred
			else if(g_mcusr_mirror & (1<<EXTRF)){
				tx_P(PSTR("external"));
			}
			else{
				tx_P(PSTR("none"));
			}
			tx_crlf();
		}
	}
		
	// Timer 1
	//TCNT1 = 0;
	//TIMSK1 |= (1<<TOIE1);
	//OCR1B = TIMER1_CLOCK/70;

	tx_P(PSTR("enabling servo input, "));
	
	servo_init();
	
	//led_on();
	
	//_delay_ms(500);
	wdt_reset();
	
	tx_P(PSTR("enabling driver, "));

	driver_enable();
	
	tx_P(PSTR("enabling interrupts, "));

	sei();

	wdt_reset();
	//_delay_ms(500);
	
	if(CONTROL_MODE == CONTROL_FIXED)
	{
		tx_P(PSTR("applying fixed throttle, "));
		psc_drive(FIXED_THROTTLE);
	}

	/*
		Set up ADC handling
	*/

	tx_P(PSTR("setting up adc, "));

	g_adc_channels[g_adc_channels_count++] = CH_CURRENT;
	g_adc_channels[g_adc_channels_count++] = CH_VOLTAGE;
	g_adc_channels[g_adc_channels_count++] = CH_CURRENT;
	g_adc_channels[g_adc_channels_count++] = CH_TEMPERATURE;

	if(CONTROL_MODE == CONTROL_SERVO1_ANALOG)
	{
		g_adc_channels[g_adc_channels_count++] = CH_CURRENT;
		g_adc_channels[g_adc_channels_count++] = CH_SERVO1;
	}
	
	if(ENABLE_MELTY){
		g_adc_channels[g_adc_channels_count++] = CH_CURRENT;
		g_adc_channels[g_adc_channels_count++] = CH_S2;
	}

	ADMUX = ADMUX_SETTINGS | g_adc_channels[g_adc_channel_counter];
	ADCSRA = ADCSRA_SETTINGS | (1<<ADSC) | (1<<ADIF);
	
	if(c_comm_mode == COMM_FET_TEST)
	{
		tx_P(PSTR("FET test starting.\r\n"));

		const uint8_t pwm = PSC_RB/10;
		struct PscControl high_active =
				{pwm, FLOAT_PSC_SB};
		struct PscControl low_active =
				{FLOAT_PSC_SA, FLOAT_PSC_SB - pwm};
		struct PscControl float_control =
				{FLOAT_PSC_SA, FLOAT_PSC_SB};

		g_usart_direct_transmit = TRUE;

		wdt_reset();

		tx_P(PSTR("nothing\r\n"));
		_delay_ms(800);

		wdt_reset();

		tx_P(PSTR("PSC_A highside\r\n"));
		_delay_ms(10);
		psc_set8(PSC_A, high_active);
		_delay_ms(800);
		psc_set8(PSC_A, float_control);

		wdt_reset();

		tx_P(PSTR("PSC_A lowside\r\n"));
		_delay_ms(10);
		psc_set8(PSC_A, low_active);
		_delay_ms(800);
		psc_set8(PSC_A, float_control);

		wdt_reset();

		tx_P(PSTR("PSC_B highside\r\n"));
		_delay_ms(10);
		psc_set8(PSC_B, high_active);
		_delay_ms(800);
		psc_set8(PSC_B, float_control);

		wdt_reset();

		tx_P(PSTR("PSC_B lowside\r\n"));
		_delay_ms(10);
		psc_set8(PSC_B, low_active);
		_delay_ms(800);
		psc_set8(PSC_B, float_control);

		wdt_reset();

		tx_P(PSTR("PSC_C highside\r\n"));
		_delay_ms(10);
		psc_set8(PSC_C, high_active);
		_delay_ms(800);
		psc_set8(PSC_C, float_control);

		wdt_reset();

		tx_P(PSTR("PSC_C lowside\r\n"));
		_delay_ms(10);
		psc_set8(PSC_C, low_active);
		_delay_ms(800);
		psc_set8(PSC_C, float_control);

		wdt_reset();

		tx_P(PSTR("Launching panic 4\r\n"));
		panic(4);
	}

	tx_P(PSTR("normal operation starting.\r\n"));

	// No more direct transmit
	g_usart_direct_transmit = FALSE;

	for(;;)
	{
		if(wdt_everything_ok())
			wdt_reset();
	
		// Check driver fault
		if(!(DRIVER_FAULT_PIN & (1<<DRIVER_FAULT_BIT)) && DEBUG_MODE)
		{
			panic(3);
		}
		
		// Check temperature
		/*if(g_temperature > 80)
			panic(4);*/
		
		// Check voltage
		/*if(g_voltage < 8000)
			panic(5);*/
		
		// Run main stuff
		if(c_comm_mode == COMM_MAINLOOP || c_comm_mode == COMM_INTERRUPT)
		{
			mainloop_bemf();
		}
		else if(c_comm_mode == COMM_SENSOR)
		{
			mainloop_sensor();
		}
		else if(c_comm_mode == COMM_NONE)
		{
			fast_busytask();
		}
		else
		{
			panic(1);
		}
	}

	return 0;
}

// =========================================================================
//                                                                INTERRUPTS
// =========================================================================

// Occurs when a byte has been received
ISR(USART_RX_vect)
{
	// Value has to be read to reset interrupt.
	uint8_t d = UDR;
	if(d == '\r' || d == ' ') // Return or space
	{
		transmit_info();
	}
}

// Occurs when transmission of a byte has completed
ISR(USART_TX_vect)
{
	// If there is stuff in the buffer, send next byte
	if(g_usart_tx_buf_size > 0)
	{
		UDR = g_usart_tx_buf[g_usart_tx_buf_start];
		USART_INDEX_INCREMENT(g_usart_tx_buf_start);
		g_usart_tx_buf_size--;
	}
}

/*ISR(TIMER1_COMPB_vect)
{
}*/

ISR(TIMER0_COMPA_vect, ISR_NOBLOCK)
{
	//OCR0A += 0x100/4;
}

/*
	At 250kHz, these overflows occur at about 1kHz or at 1ms intervals.
	This is always running.
*/
ISR(TIMER0_OVF_vect)
{
	wdt_run_timers();
	
	if(g_runtime_ms < 0xffff)
		g_runtime_ms++;

	if(ENABLE_SERVO2_INPUT && servo2_state != 2)
	{
		//DACH++;
		if((servo2_state == 0 && timer0_ovf_counter_servo2 >= 30)
		|| (servo2_state == 1 && timer0_ovf_counter_servo2 >= 3)){
			// Input is dead
			servo2_report_invalid(3);
			g_target_throttle = 0;
			
			if(ENABLE_SERVO1_INPUT)
			{
				servo2_stop_input();
				servo1_start_input();
			}
		}
		else{
			timer0_ovf_counter_servo2++;
		}
	}

	if(ENABLE_SERVO1_INPUT && servo1_state != 2)
	{
		//DACH++;
		if((servo1_state == 0 && timer0_ovf_counter_servo1 >= 30)
		|| (servo1_state == 1 && timer0_ovf_counter_servo1 >= 3)){
			// Input is dead
			servo1_report_invalid(3);
			g_target_throttle = 0;
			
			if(ENABLE_SERVO2_INPUT)
			{
				servo1_stop_input();
				servo2_start_input();
			}
		}
		else{
			timer0_ovf_counter_servo1++;
		}
	}

	if(c_ledmode == LEDMODE_RX)
	{
		static uint8_t ledcount = 0;
		if(servo_input_ok())
		{
			ledcount++;
			if(ledcount < 128)
				led_on();
			else
				led_off();

			// Count shorter time if input is not near zero
			if(servo2_valid && (servo2_value < 128-20 || servo2_value > 128+20))
			{
				if(ledcount > 128+64)
					ledcount = 64;
			}
		}
		else
			led_on();
	}

	// The following code is slow; enable interrupts
	sei();
	
	if(CONTROL_MODE == CONTROL_SERVO_MELTY_ACCEL)
	{
		int16_t result;
		ATOMIC_BLOCK(ATOMIC_RESTORESTATE){
			result = g_accel;
		}
#define pi 3.14159265
#define r 0.041
#define CORR_FACTOR 1.08
		// result is about 180 at 20r/s
		// r/s = sqrt(a * r) / 2 / pi / r
		// s/r = 2 * pi * r / sqrt(a * r)
		// a = result * 5.0V / 0.01V/g / 256 * 9.81m/s^2
		// s/r = 2 * pi * r / sqrt(result * 5.0V / 0.01V/g / 256 * r * 9.81m/s^2)
		// s/r = 2 * pi * r / sqrt(5.0V / 0.01V/g / 256 * r * 9.81m/s^2) / sqrt(result)
		
		// ticks/r = 31250ticks/s * 2 * pi * r / sqrt(5.0V / 0.01V/g / 256 * r * 9.81m/s^2) / sqrt(result)
		// ticks/r = 31250ticks/s * 2 * pi * r / sqrt(5.0V / 0.01V/g / 256 * r * 9.81m/s^2 / (2^32/256)) / sqrt(result * (2^32/256))
		//int32_t tickspr = (uint32_t)((double)TIMER1_CLOCK * 2.0 * pi * r / sqrt(5.0 / 0.01 / 256 * r * 9.81 / (pow(2,32)/256)) * CORR_FACTOR) / isqrt(result * (pow(2,32)/256));
		
		// s/r = 2 * pi * r / sqrt(5.0V / 0.01V/g / 256 * r * 9.81m/s^2) / sqrt(result)
		// r/s = sqrt(5.0V / 0.01V/g / 256 * r * 9.81m/s^2) / (2 * pi * r) * sqrt(result)
#define full_angle 65536.0
#define tick_time 1000.0
		int32_t angle_add =
			(uint32_t)(
				(double)(full_angle/tick_time)
				* sqrt(5.0 / 0.01 / 4096 * r * 9.81 * pow(2,16))
				* CORR_FACTOR / (2.0 * pi * r)
			)
			* isqrt(result * pow(2,16))
			/ (uint32_t)pow(2,16);
		
		g_angle_robot_re_ground += angle_add;
		g_dir_re_robot -= angle_add;
	}

	// Melty LED at PC7
	if(ENABLE_MELTY)
	{
#if 1
		if(g_comm_phase_length < TIMER1_CLOCK/(1000/60)/6)
		{
			if(g_dir_re_robot < 65536/6 ||
					g_dir_re_robot > 65535 - 65535/6)
				PORTC |= (1<<7);
			else
				PORTC &= ~(1<<7);

			/*if(g_angle_robot_re_ground < 65536/16 ||
					g_angle_robot_re_ground > 65536 - 65536/16)
				PORTC ^= (1<<7);
			else
				PORTC ^= (1<<7);*/
		}
		else
		{
			if(wdt_everything_ok()){
				static uint8_t c = 0;
				c++;
				if(c < 128)
					PORTC |= (1<<7);
				else
					PORTC &= ~(1<<7);
			} else if(servo_input_with_wdt_ok()){
				static uint8_t c = 0;
				c++;
				c++;
				c++;
				if(c < 128)
					PORTC |= (1<<7);
				else
					PORTC &= ~(1<<7);
			} else {
				PORTC |= (1<<7);
			}
		}
#endif
#if 0
		if(g_accel >= 30)
			PORTC |= (1<<7);
		else
			PORTC &= ~(1<<7);
#endif
	}
}

ISR(INT1_vect)
{
	if(ENABLE_SERVO2_INPUT)
	{
		if(servo2_state == 0)
		{
			// SERVO_IN_2 = INT1 was low, now rising
			// Reset counter
			TCNT0 = 0;
			// Clear overflow flag
			TIFR0 |= (1<<TOV0);
			// Clear overflow count
			timer0_ovf_counter_servo2 = 0;

			// Falling edge on INT1 generates an interrupt request
			EICRA |= (1<<ISC11);
			EICRA &= ~(1<<ISC10);
			// Clear interrupt
			EIFR |= (1<<INTF1);
			// High state
			servo2_state = 1;

			//DACH = 50;
			return;
		}
		
		// SERVO_IN_2 = INT1 was high, now falling

		//DACH = 100;

		uint8_t value;
		uint8_t error_code = 0;
		if(timer0_ovf_counter_servo2 == 0)
		{
			value = 0;
			// Too short
			if(TCNT0 < 200)
				error_code = 1;
		}
		else if(timer0_ovf_counter_servo2 >= 2)
		{
			value = 255;
			// Too long
			if(TCNT0 > 50)
				error_code = 2;
		}
		else{
			value = TCNT0;
		}
		
		if(ENABLE_SERVO1_INPUT)
		{
			servo2_stop_input();
			servo1_start_input();
		}
		else
		{
			servo2_start_input();
		}
		
		// If servo pulse was <<1ms or >>2ms
		if(error_code)
		{
			servo2_report_invalid(error_code);
		}
		else
		{
			// Now we have a valid value
			
			// Enable interrupts while processing stuff
			sei();

			servo2_report_valid(value);
		}
	}
}

ISR(INT2_vect)
{
	if(ENABLE_SERVO1_INPUT)
	//if(0)
	{
		if(servo1_state == 0)
		{
			// SERVO_IN_1 = INT2 was low, now rising
			// Reset counter
			TCNT0 = 0;
			// Clear overflow flag
			TIFR0 |= (1<<TOV0);
			// Clear overflow count
			timer0_ovf_counter_servo1 = 0;

			// Falling edge on INT2 generates an interrupt request
			EICRA |= (1<<ISC21);
			EICRA &= ~(1<<ISC20);
			// Clear interrupt
			EIFR |= (1<<INTF2);
			// High state
			servo1_state = 1;

			//DACH = 50;
			return;
		}
		
		// SERVO_IN_2 = INT1 was high, now falling

		//DACH = 100;

		uint8_t value;
		uint8_t error_code = 0;
		if(timer0_ovf_counter_servo1 == 0)
		{
			value = 0;
			// Too short
			if(TCNT0 < 200)
				error_code = 1;
		}
		else if(timer0_ovf_counter_servo1 >= 2)
		{
			value = 255;
			// Too long
			if(TCNT0 > 50)
				error_code = 2;
		}
		else{
			value = TCNT0;
		}
		
		if(ENABLE_SERVO2_INPUT)
		{
			servo1_stop_input();
			servo2_start_input();
		}
		else
		{
			servo1_start_input();
		}
		
		// If servo pulse was <<1ms or >>2ms
		if(error_code)
		{
			servo1_report_invalid(error_code);
		}
		else
		{
			// Now we have a valid value
			
			// Enable interrupts while processing stuff
			sei();

			servo1_report_valid(value);
		}
	}
}

ISR(PSC0_EC_vect)
{
	//DACH = 255;
	//led_on();

	/*if(c_comm_mode == COMM_SENSOR)
	{
		update_sensor_state();
		psc_apply();
	}*/

	if(c_comm_mode == COMM_INTERRUPT || c_comm_mode == COMM_MAINLOOP)
	{
		/*
			End-of-Cycle Interrupt of PSC0 (to which PSC1 and PSC2 is synced to.
			
			This is launched at the end of every PSC cycle, which means the point
			of PWM that is at the middle of lowtime.
		*/
		
		uint8_t floating_psc = g_float_psc;
		uint8_t float_ph_state;
		if     (floating_psc == 0) float_ph_state = ACSR & (1<<2);
		else if(floating_psc == 1) float_ph_state = ACSR & (1<<1);
		else                       float_ph_state = ACSR & (1<<0);
		float_ph_state = float_ph_state ? 1:0;

		/*
			Variables:
			float_ph_state
			g_float_ph_last_state
			g_float_ph_last_stable_state
			g_float_ph_stability_counter
		*/
		uint8_t floating_psc_last_state = g_float_ph_last_state;
		g_float_ph_last_state = float_ph_state;
		if(float_ph_state != floating_psc_last_state){
	#if 0
			if(g_comm_phase_length < TIMER1_CLOCK/(15000/60)/6)
				g_float_ph_stability_counter = 1;
			else
				g_float_ph_stability_counter = 3;
	#endif
	#if 1
			if(g_comm_phase_length < TIMER1_CLOCK/(15000/60)/6)
				g_float_ph_stability_counter = 1;
			else if(g_comm_phase_length < TIMER1_CLOCK/(5000/60)/6)
				g_float_ph_stability_counter = 3;
			else
				g_float_ph_stability_counter = 6;
	#endif
	#if 0
				g_float_ph_stability_counter = 1;
	#endif
		}
		else{
			if(g_float_ph_stability_counter > 0){
				g_float_ph_stability_counter--;
				if(g_float_ph_stability_counter == 0){
					g_float_ph_last_stable_state = float_ph_state;
				}
			}
		}
		
		/*if(g_float_ph_last_stable_state == 255)
			DACH = 128;
		else
			DACH = g_float_ph_last_stable_state * 255;*/
		
		//DACH = float_ph_state * 255;
	}

	if(c_comm_mode == COMM_INTERRUPT)
	{
		static uint8_t k = 0;
		static uint8_t wait_initial = 0;

		uint8_t rising;
		if(g_reverse)
			rising = !g_comm_table[g_comm_index].rising;
		else
			rising = g_comm_table[g_comm_index].rising;

		if(wait_initial){
			if((g_float_ph_last_stable_state == !rising
					|| TCNT1 >= TIMER1_CLOCK/30)
					&& g_float_ph_stability_counter == 0){
				wait_initial = 0;
			}
		}
		else{
			if((g_float_ph_last_stable_state == rising
					|| TCNT1 >= TIMER1_CLOCK/30)
					&& g_float_ph_stability_counter == 0){
				// TODO: check this line
				if(k >= 1 + (g_comm_phase_length/1024)){
					uint16_t t = TCNT1;
					TCNT1 = 0;
					TIFR1 |= (1<<TOV1);
					if(TIFR1 & (1<<TOV1)){
						g_comm_phase_length = UINT16_MAX;
					}
					else{
						uint16_t l = g_comm_phase_length;
						g_comm_phase_length = l - l/4 + t/4;
					}
					
					if(g_reverse){
						g_comm_index = get_next_comm_index_reverse(g_comm_index);
					}
					else{
						g_comm_index = get_next_comm_index(g_comm_index);
					}
					psc_select_from_table();
					psc_apply();

					if(g_comm_phase_length < TIMER1_CLOCK/(2000/60)/6)
						wait_initial = 1;

					if(g_comm_index == 0)
						led_on();
					else
						led_off();
					k = 0;
				}
				else{
					k++;
				}
			}
			else{
				//if(k < 255) k++;
			}
		}
	}

	PIFR0 |= (1<<PEOP0);

	//DACH = 0;
	//led_off();
}


